#############################################################################
#############################################################################
#
# latexSmart.tcl (called from latex.tcl)
#
# Smart quotes, dots, subscripts, and superscripts
#
#############################################################################
#
# Original author unknown
#
# Maintainer:  Tom Scavo <trscavo@syr.edu>
#
#############################################################################
#############################################################################

proc latexSmart.tcl {} {}

#--------------------------------------------------------------------------
# Smart quotes:
#--------------------------------------------------------------------------

proc smartDQuote {} {
	global TeXmodeVars
	if {[isSelection]} { deleteSelection }
	if { !$TeXmodeVars(smartQuotes) || [literalChar] } { insertText {"}; return }
	if {[leftQ]} {
		insertText {``}
	} else {
		insertText {''}
	}
}

proc smartQuote {} {
	global TeXmodeVars
	if {[isSelection]} { deleteSelection }
	if { !$TeXmodeVars(smartQuotes) || [literalChar]  } { insertText {'}; return }
	if {[leftQ]} {
		insertText {`}
	} else {
		insertText {'}
	}
}

proc leftQ {} {
	if { [getPos] == 0 } { return 1 };
	set q [lookAt [expr [getPos]-1]]
	case $q in {
		{\t}  {return 1}
		{(}  {return 1}
		{\{}  {return 1}
		{[}  {return 1}
		{<}  {return 1}
		{\ } {return 1}
		{\r} {return 1}
	}
	return 0
}

#--------------------------------------------------------------------------
# Smart dots:
#--------------------------------------------------------------------------

# proc smartDots {} {
# 	global TeXmodeVars
# 	if {[isSelection]} { deleteSelection }
# 	if { !$TeXmodeVars(smartDots) || [literalChar] } { insertText {.}; return }
# 	# Labels contain literal dots:
# 	set pat {\\(label|(page|eq)?ref|bibitem|(no)?cite)(\[[^][]*\])?\{}
# 	if { [findPatJustBefore "$pat" "${pat}\[\]\[()`'.,:;?!a-zA-Z0-9/@*-\]*\$"] != "" } {
# 		insertText "."
# 		return
# 	}
# 	# Filenames contain literal dots:
# 	set pat {\\(usepackage|input|include(only)?|documentclass|bibliography(style)?|LoadClass|RequirePackage|begin\{filecontents\})(\[[^][]*\])?\{}
# 	if { [findPatJustBefore "$pat" "${pat}\[.:a-zA-Z_0-9/-\]*\$"] != "" } {
# 		insertText "."
# 		return
# 	}
# 	if {[lookAt [expr [set endPos [getPos]]-1]] == "."} {
# 		if {[lookAt [set begPos [expr $endPos-2]]] == "."} {
# 			replaceText $begPos $endPos "\\ldots"
# 		} else {
# 			insertText "."
# 		}
# 	} else {
# 		insertText "."
# 	}
# }
proc smartDots {} {
	global TeXmodeVars
	if {[isSelection]} { deleteSelection }
	if { !$TeXmodeVars(smartDots) || [literalChar] } { insertText {.}; return }
	if {[lookAt [expr [set endPos [getPos]]-1]] == "."} {
		if {[lookAt [set begPos [expr $endPos-2]]] == "."} {
			replaceText $begPos $endPos "\\ldots"
		} else {
			insertText "."
		}
	} else {
		insertText "."
	}
}

#--------------------------------------------------------------------------
# Smart subscripts and superscripts:
#--------------------------------------------------------------------------

proc smartSubscripts {} {
	smartScripts {_}
}

proc smartSuperscripts {} {
	smartScripts {^}
}

proc smartScripts {char} {
	if {[isSelection]} { deleteSelection }
	if {[literalChar]} {
		insertText $char
		return
	}
	# Filenames contain literal underscores:
	set pat {\\(usepackage|input|include(only)?|documentclass|bibliography(style)?|LoadClass|RequirePackage|begin\{filecontents\})(\[[^][]*\])?\{}
	if { [findPatJustBefore "$pat" "${pat}\[.:a-zA-Z0-9/^_-\]*\$"] != "" } {
		insertText $char
		return
	}
	if { $char == {_} } { subscript } { superscript }
}

#--------------------------------------------------------------------------
# Escapes and exceptions:
#--------------------------------------------------------------------------

proc escapeSmartStuff {} {
	if {![isSelection]} {
		set pos [getPos]
		# Escape double quotes:
		if { $pos > 1 } {
			set pos2 [expr $pos - 2]
			if { [getText $pos2 $pos] == "''" } {
				replaceText $pos2 $pos {"}
				return
			} elseif { [getText $pos2 $pos] == "``" } {
				replaceText $pos2 $pos {"}
				return
			}
		}
		# Escape single quote:
		if { $pos > 0 } {
			set pos1 [expr $pos - 1]
			if { [lookAt $pos1] == "`" } {
				backSpace
				insertText {'}
				return
			}
		}
		# Escape dots:
		if { $pos > 5 } {
			set pos6 [expr $pos - 6]
			if { [getText $pos6 $pos] == "\\ldots" } {
				replaceText $pos6 $pos {...}
				return
			}
		}
		# Escape underscore:
		if { $pos > 1 && [maxPos] > [expr $pos + 1] } {
			set begPos [expr $pos - 2]
			set endPos [expr $pos + 2]
			if { [getText $begPos $endPos] == "_\{\}" } {
				replaceText $begPos $endPos {_}
				return
			}
		}
		# Escape caret:
		if { $pos > 1 && [maxPos] > [expr $pos + 1] } {
			set begPos [expr $pos - 2]
			set endPos [expr $pos + 2]
			if { [getText $begPos $endPos] == "^\{\}" } {
				replaceText $begPos $endPos {^}
				return
			}
		}
	}
	backSpace
}

