# Note that if Alpha upgrades to Tcl 8.0, these procs should be replaced
# with simple wrappers around the Tcl procs 'file mkdir' 'file copy' etc.
# (the built in Tcl 8.0 ones are more robust and cope better with the
# Mac filesystem than Alpha's procedures)
#================================================================================

###########################################################################
#  better-cp-mv.tcl  -- modification of your routines, by Mark Nagata
#  for Alpha 5.72,  1/04/94
###########################################################################
proc cp args {
    if {[llength $args] < 2} {
        error "usage: cp <file1> <file2>\r       cp <file1> .... <dir>"
    }
    set dir [lindex $args end]
    if {![regexp {:} $dir] && $dir != ""} {
        set dir ":$dir"
    }
    if {[regexp {:$} $dir]} {
        set dir [string trimright $dir {:}]
    }
    set args [lreplace $args end end]
    set files {}
    foreach arg $args {
        eval lappend files [glob -nocomplain $arg]
    }
	movecopy copy $dir $files
}

proc mv args {
    if {[llength $args] < 2} {
        error "usage: mv <file1> <file2>\r       mv <file1> .... <dir>"
    }
    set dir [lindex $args end]
    if {![regexp {.*[^:]} $dir dir]} {
        set dir [string range $dir 1 end]
    }
    if {![regexp {:} $dir] && $dir != ""} {
        set dir [concat :$dir]}
    set args [lreplace $args end end]
    set files {}
    foreach arg $args {
        eval lappend files [glob -nocomplain $arg]
    }
	movecopy move $dir $files
}

proc movecopy {cmd dir files} {
    set report ""
    if {[llength $files] == 1} {
        set f [lindex $files 0]
        if {[file exists $dir]} {
            set targ $dir:[file tail $f]
            append report "$f >-> $targ \r"
            ${cmd}File $f $targ
        } else {
            append report "$f >-> $dir \r"
            ${cmd}File $f $dir
        }
    } else {
        foreach f $files {
        	message [file tail $f]
            set targ $dir:[file tail $f]
            if {[catch {${cmd}File $f $targ} that]} {
                append report "Error trying to $cmd '$f': $that\r"
            } else {
            	append report "$f >-> $targ \r"
            }
        }
    }
    echo [string trimright $report]
}

proc rm args {
	set opts(-r) 0
	getOpts
	set files {}
	foreach arg $args {
		eval lappend files [glob -nocomplain $arg]
	}
	__rm $opts(-r) $files
}

proc __rm {recurse names} {
	foreach f $names {
		if {[file isdir $f]} {
			if $recurse {
				__rm $recurse [glob -nocomplain ${f}:*]
			} 
			rmdir $f
		} else {			
			message [file tail $f]		
			removeFile $f				
		}			
	}
}

proc textToAlpha {{dir ""}} {
	set num 0
	if {![string length $dir]} {
		set dir [get_directory -p "Creators to 'ALFA':"]
	}

	foreach f [glob -nocomplain "$dir:*"] {
		if {[file isfile $f] && ([getFileType $f] == "TEXT") && ([getFileSig $f] != "ALFA")} {
			message $f
			setFileInfo $f creator ALFA
			incr num
		} elseif {[file isdir $f]} {
			incr num [textToAlpha $f]
		}
	}
	message "Converted $num files"
	return $num
}
