## -*-Tcl-*-
 # ###################################################################
 #  Alpha - new Tcl folder configuration
 # 
 #  FILE: "alphaDefinitions.tcl"
 #                                    created: 26/9/97 {11:25:02 am} 
 #                                last update: 8/12/97 {10:19:39 pm} 
 #  Author: Vince Darley
 #  E-mail: <darley@fas.harvard.edu>
 #    mail: Division of Engineering and Applied Sciences, Harvard University
 #          Oxford Street, Cambridge MA 02138, USA
 #     www: <http://www.fas.harvard.edu/~darley/>
 #  
 # Reorganisation carried out by Vince Darley with much help from Tom 
 # Fetherston, Johan Linde and suggestions from the Alpha-D mailing list.  
 # Alpha is shareware; please register with the author using the register 
 # button in the about box.
 #  
 # ###################################################################
 ##

proc alpha::getDefinitions {} {}
namespace eval keys {}
namespace eval win {}

# Default settings of some global parameters (not preferences settings)
# These will all be over-written by user-defs if set.
set resumeRevert 0
set prefixString ">\ "
set suffixString "\ <--"
# mapping of windows to current modes.
set win::Modes("") ""
# keep count of number of dirty windows.
set win::NumDirty 0
set win::Current ""
# For mark stack.
set markName 0
set markStack ""
set winMenu "263"
set globalMenus_curr {filesetMenu internetConfigMenu eudoraMenu}
set keyboard "U.S."
set oldkeyboard ""

win::SetProportions

lunion ftpSigs Arch FTCh Woof
lunion texSigs OTEX *TEX *XeT MPS*
lunion browserSigs MOSS MSIE dogz OlG1 HTVW
lunion httpDownloadSigs MOSS MSIE dogz OlG1 Geni
lunion tclshSigs WIsH TclL Geni

array set keys::specialProcs { 
	"Next Stop" "ring::+"
	"Next Stop Or Indent" "bind::IndentOrNextstop"
	"Complete" "bind::Completion"
	"Complete Or Tab" "bind::TabOrComplete"
	"Prev Stop" "ring::-"
	"Real Tab" "insertActualTab"
	"nth Stop" "ring::nth"
	"Clear All Stops" "ring::clear"
}

array set keys::specialBindings {
	"Complete" "<O/c"
	"Prev Stop" "<U/c"
	"Real Tab" "<I/c"
	"nth Stop" "<B/c"
	"Clear All Stops" "<U<B/c"
	"Next Stop" ""
	"Complete Or Tab" ""
	"Next Stop Or Indent" "/c"
}

# Note: the Mercution MDEF can only handle icon-suite resources
# with id's from 208 to 208+255 = 463.  Hence many of the little
# icons which Alpha contains cannot appear in menus.  You could
# of course do a little hacking....
set alpha::_icons {
	{DanR "Think Reference" 265}
	{OTEX "OzTeX" 266}
	{*TEX "TeXtures" 267}
	{XXXX "LaTeX" 270}
	{*XeT "CMacTeX" 272}
	{TeX+ "DirectTeX Pro" 299}
	{CWIE "Codewarrior" 268}
	{dogz "Cyberdog" 281}
	{Vbib "BibTeX" 282}
	{SLab "Scilab" 283}
	{IGR0 "Igor Pro" 284}
	{JAVC "Apple Applet Viewer" 285}
	{MOSS "Netscape Navigator" 293}
	{MOSS "Netscape Communicator" 294}
	{MSIE "Microsoft Internet Explorer" 295}
	{OlG1 "MacLynx" 296}
	{DanR "Think Ref Viewer" 310}
	{gsVR "Ghostview" 311}
	{PnLF "Finger" 313}
	{RZMI "MakeIndex" 314}
	{FTCh "Fetch" 315}
	{TGE+ "Tarmac" 316}
	{Gzip "Gzip" 317}
	{DStf "DropStuff" 318}
	{SITx "StuffIt Expander" 319}
	{ALTV "Programmer's Assistant" 400}
	{MPAD "Mupad" 411}
	{GPSE "Gnuplot" 415}
	{Vodo "VOODOO" 500}
}

namespace eval remote {}

ensureset "remote::site(Alpha's Home)" \
  "ftp://www.cs.umd.edu/pub/faculty/keleher/Alpha/packages/"
ensureset "remote::site(Alpha's Upload site)" \
  "ftp://www.cs.umd.edu/pub/faculty/keleher/Alpha/up/"
ensureset "remote::site(Vince's Alpha-D www site)" \
  "http://www.santafe.edu/~vince/pub/Alpha/"

ensureset remote::listing {}


set "keyboards(Australian)" {
 {1234567890-=[];'\`,./}
 {!@#$%^&*()_+{}:"|~<>?}
 <U/[
 <U/]
}
set "keyboards(Brasil)" {
 {1234567890'+~\<,.-}
 {!"#$%&/()=?*`^|>;:_}
 <I<U/8
 <I<U/9
}
set "keyboards(British)" {
 {1234567890-=[];'\`,./}
 {!@$%^&*()_+{}:"|~<>?}
 <U/[
 <U/]
}
set "keyboards(Canadian - CSA)" {
 {/1234567890-=^;,.}
 {\!@#$%?&*()_+:'"}
 <I/7
 <I/8
}
set "keyboards(Canadian - ISO)" {
 {1234567890-=^;,.}
 {!"#$%?&*()_+:'.}
 <I/7
 <I/0
}
set "keyboards(Canadian - French)" {
 {<1234567890-='[;`/,.}
 {>!@#$%?&*()_+"]:^|<>}
 <I/[
 <I<U/[
}
set "keyboards(Danish)" {
 {$1234567890+'<,.-}
 {!"#%&/()=?`^*>;:_}
 <I<U/8
 <I<U/9
}
set "keyboards(Dutch)" {
 {1234567890-=[];'\`,./}
 {!@#$%^&*()_+{}:"|~<>?}
 <U/[
 <U/]
}
set "keyboards(Espaol - ISO)" {
 {1234567890'`+<,.-}
 {!"$%&/()=?^*>;:_}
 <I/
 <I/
}
set "keyboards(Finnish)" {
 {1234567890+'<,.-}
 {!"#%&/()=?`^*>;:_}
 <I<U/8
 <I<U/9
}
set "keyboards(Flemish)" {
 {@&"'(!)-^$`<,;:=}
 {#1234567890_*%>?./+}
 <I/(
 <I/)
}
set "keyboards(French)" {
 {@&"'(!)-^$`<,;:=}
 {#1234567890_*%>?./+}
 <I/(
 <I/)
 {'(' 0x17 ')' 0x1b}
}
set "keyboards(French - numerical)" {
 {@&"'(!)-^$`<,;:=}
 {#1234567890_*%>?./+}
 <I/(
 <I/)
 {'(' 0x17 ')' 0x1b}
}
set "keyboards(German)" {
 {^1234567890+#<,.-}
 {!"$%&/()=?`*^>;:_}
 <I/8
 <I/9
}
set "keyboards(Italian)" {
 {@&"'()-=$<,;:}
 {#1234567890_+^*%>?./!}
 <I/(
 <I/)
}
set "keyboards(Norwegian)" {
 {'1234567890+@<,.-}
 {!"#$%&/()=?`^*>;:_}
 <I<U/8
 <I<U/9
}
set "keyboards(Spanish)" {
 {[1234567890-=`;'<,.}
 {]!#$%/&*()_+:">?}
 <I<U/<
 <U/[
}
set "keyboards(Swedish)" {
 {1234567890+'<,.-}
 {!"#%&/()=?`^*>;:_}
 <I<U/8
 <I<U/9
}
set "keyboards(Swiss French)" {
 {1234567890'^$<,.-}
 {+"*%&/()=?`!>;:_}
 <I/8
 <I/9
}
set "keyboards(Swiss German)" {
 {1234567890'^$<,.-}
 {+"*%&/()=?`!>;:_}
 <I/8
 <I/9
}
set "keyboards(U.S.)" {
 {1234567890-=[];'\`,./}
 {!@#$%^&*()_+{}:"|~<>?}
 <U/[
 <U/]
}
