## -*-Tcl-*-
 # ###################################################################
 #  Alpha - new Tcl folder configuration
 # 
 #  FILE: "alphaMenus.tcl"
 #                                    created: 21/9/97 {10:24:37 pm} 
 #                                last update: 20/12/97 {7:17:45 pm} 
 #  Author: Vince Darley
 #  E-mail: <darley@fas.harvard.edu>
 #    mail: Division of Engineering and Applied Sciences, Harvard University
 #          Oxford Street, Cambridge MA 02138, USA
 #     www: <http://www.fas.harvard.edu/~darley/>
 #  
 # Reorganisation carried out by Vince Darley with much help from Tom 
 # Fetherston, Johan Linde and suggestions from the Alpha-D mailing list.  
 # Alpha is shareware; please register with the author using the register 
 # button in the about box.
 #  
 # Description:
 # 
 #  Initialises variables which contain the global menus.  If you 
 #  use the smarterSource package, you can over-ride these quite
 #  easily.  This file is sourced by the procedure 'menu::buildSome'
 #  which handles menu building.
 # ###################################################################
 ##
#  File menu  #
set menu::items(File) {
	"/Nnew"
	"/Oopen"
	"<S/Wclose"
	"<S<O<U/WcloseFloat"
	"<S<I<O/WcloseAll"
	"(-"
	"<S/Ssave"
	"<S<B<O/SsaveUnmodified"
	"<S<I<O/SsaveAll"
	"<SsaveACopyAs"
	"<S<UrenameTo"
	"<SsaveAs"
	"<E<SrevertToBackup"
	"<Srevert"
	"(-"
	"<S/Y<Oshell"
	"<S/Y<U<Ocalculator"
	"<S/Y<B<OtoolserverShell"
	"(-"
	"pageSetup"
	"/P<Sprint"
	"(-"
	"/Qquit"
}

#  Edit menu  #
set menu::items(Edit) {
	"/Zundo"
	"/Z<I<Oredo"
	"/Lload"
	"(-"
	"/X<Scut"
	"/C<Scopy"
	"/V<Spaste"
	"/A<SselectAll"
	"/A<S<I<OselectParagraph"
	"clear"
	"(-"
	"/`<Stwiddle"
	"/`<S<I<OtwiddleWords" 
	"(-" 
	{/[<SshiftLeft} 
	{/[<S<I<OshiftLeftSpace} 
	{/]<SshiftRight} 
	{/]<S<I<OshiftRightSpace} 
	"/Bbalance" 
}

#  Text menu  #
set menu::items(Text) {
	"/I<SfillParagraph"
	"/I<S<O<IwrapParagraph"
	"/I<S<O<I<UsentenceParagraph"
	"/U<S<OfillRegion"
	"/U<S<I<OwrapRegion"
	"/U<S<O<I<UsentenceRegion"
	"<E<SparagraphToLine"
	"<SlineToParagraph"
	"(-"
	"<SreverseSort"
	"<SsortLines"
	"<S/L<O<IspellcheckWindow"
	"<S/L<O<I<BspellcheckSelection"
	"(-"
	"zapInvisibles"
	"<SspacesToTabs"
	"<StabsToSpaces"
	"<S<EindentRegion"
	"<SindentLine"
	"<E<SdowncaseRegion"
	"<SupcaseRegion"
	"(-"
	{menu -n Strings {
		"<SremovePrefix"
		"<SinsertPrefix"
		"<S<EremoveSuffix"
		"<SinsertSuffix"
		"setPrefix"
		"setSuffix"
		}}
	"/D<ScommentLine"
	"/D<S<I<OuncommentLine"
	"<SuncommentBox"
	"<ScommentBox"
	"<S<EuncommentParagraph"
	"<ScommentParagraph"
	"(-"
	"/K<I<OgotoFunc"
	"/K<O<BgotoFileMark"
	"/=markHilite"
	{menu -n namedMarks -p namedMarkProc {
		"/K<Oset"
		"goto"
		"remove"
		"(-"
		"sort"
		"sortByPosition"}}
	{menu -n unnamedMarks -p unnamedMarkproc {
		"set"
		"exchangePointAndMark"}}
}

#  Search menu  #
set menu::items(Search) {
	"/F<Sfind"
	"/F<S<I<OsearchStart"
	"/G<SfindAgain"
	"/G<S<I<OfindAgainBackward"
	"/MfindInNextFile"
	"/E<SenterSearchString"
	"/E<S<I<OenterReplaceString"
	"(-"
	"/S<S<BquickFind"
	"/S<S<B<IquickFindRegexp"
	"/R<BreverseQuickFind"
	"(-"
	"/Rreplace"
	"/Hreplace&FindAgain"
	"<S/R<O<IreplaceAll"
	"<S/R<O<I<BreplaceInFileset"
	"(-"
	"/,<BpushPosition"
	"/.<BpopPosition"
	"/G<IgotoLine"
}
set menu::proc(Search) ""

#  Utils menu  #
set menu::proc(fileUtils) menu::fileUtils
set menu::proc(winUtils) menu::fileUtils
set menu::items(fileUtils) {
	"fileRemove"
	"fileInfo"
	"textToAlpha"
}
set menu::items(winUtils) {
	"insertPathName"
	"insertFile"
	"wordCount"
}

set menu::items(Utils) {
	{menu -n fileUtils -p menu::fileUtils {}}
	{menu -n winUtils -p menu::fileUtils {}}
	{menu -n asciiEtc {
		"quoteChar"
		"(-"
		"keyCode"
		"keyAscii"
		"getAscii"
		"insertAscii"}
	}
	"(-"
	"/M<I<OmatchingLines" 
	"gotoMatch" 
	"/M<BnextMatch"
	"(-" 
	"sendUrl"
	"/jcmdDoubleClick"
}
set menu::proc(Utils) ""
lappend menu::which_subs(Utils) "winUtils" "fileUtils"

#  Config menu  #
set menu::items(Config) {
	{menu -n "currentMode" -p mode::menuProc {}}
	{menu -n global -p menu::generalProc {}}
	"/p<UsuffixMappings"
	{menu -n "packages" -p package::menu {}}
	{menu -m -n alphaDownloads -p remote::menu {}}
	"helperApps"
	"(-"
	"/kdescribeBinding"
	"viewSavedSetting"
	"removeSavedSetting"
	"(-"
	"/lsetFontsTabs"
	{menu -n redefineColors -p menu::colorProc {
		foreground
		background
		"(-"
		blue
		cyan
		green
		magenta
		red
		white
		yellow
		"(-"
		color_9
		color_10
		color_11
		color_12
		color_13
		color_14
		color_15}}
}
set menu::proc(Config) ""
lappend menu::which_subs(Config) "global" "mode" packages alphaDownloads
