## -*-Tcl-*-
 # ###################################################################
 #  Alpha - new Tcl folder configuration
 # 
 #  FILE: "coreFixes.tcl"
 #                                    created: 31/7/97 {2:09:16 am} 
 #                                last update: 20/12/97 {6:31:54 pm} 
 #  Author: Vince Darley
 #  E-mail: <darley@fas.harvard.edu>
 #    mail: Division of Engineering and Applied Sciences, Harvard University
 #          Oxford Street, Cambridge MA 02138, USA
 #     www: <http://www.fas.harvard.edu/~darley/>
 #  
 # Reorganisation carried out by Vince Darley with much help from Tom 
 # Fetherston, Johan Linde and suggestions from the Alpha-D mailing list.  
 # Alpha is shareware; please register with the author using the register 
 # button in the about box.
 #  
 # This file contains Tcl procs which wrap around or replace
 # core (hard-coded) Alpha procs to fix some bugs they may have.
 # Sadly most core Alpha bugs can't be fixed in this way.
 # 
 # Ultimately, one hopes, these bugs will be fixed and these procs
 # can be removed...
 # ###################################################################
 ##

# Note: All other procs in this file start with a ';' to avoid
# being noted in the auto_index file.  Alpha sources this file
# at startup.
proc alpha::fixCoreBugs {} {}

# Fixes two bugs: the message in the status window was incorrect (shows
# the search, not replace string.  Also a replace string of nothing was
# rejected.

rename enterReplaceString ""
;proc enterReplaceString {} {
	set t [getSelect]
	replaceString $t
	message "Entered replace '$t'"
}


# Setting fonts and tabs doesn't need to dirty the window
rename setFontsTabs __setFontsTabs
;proc setFontsTabs {} {
	set d [winDirty]
	__setFontsTabs
	if {!$d && [winDirty]} {
		setWinInfo dirty 0
	}
}

# so any selections present are maintained
rename centerRedraw __centerRedraw
;proc centerRedraw {} {
	lappend selectionEndPoints [getPos] [selEnd]
	__centerRedraw
	eval select $selectionEndPoints 
}

# so any selections present are maintained
rename insertToTop __insertToTop
;proc insertToTop {} {
	lappend selectionEndPoints [getPos] [selEnd]
	__insertToTop
	eval select $selectionEndPoints 
}

# if you select a directory from inside it, it has a ':', if you select
# from outside, it doesn't have a colon.  There is another problem, which
# is that Alpha won't let you select a volume, only a folder within a 
# volume, but I haven't fixed that here.
rename get_directory __get_directory
;proc get_directory {args} {
	set dir [eval __get_directory $args]
	regsub {:$} $dir {} dir
	return $dir
}
if {[info tclversion] < 8.0} {
# old version is a bit picky
if {![string length [info commands __cd]]} {
	rename cd __cd
}
;proc cd args {
	if {$args == ".."} { set args "::" }
	if {$args == "."} { set args ":" }
	if {[llength $args]} {
		set path [string trim [eval list $args] "		\{\}"]
		if ![regexp {:$} $path] { append path ":" }
		if {![file isdir $path] && [file isdir [pwd]$path]} {
			set path ":$path"
		}
		__cd $path
	} else {
		global HOME
		__cd $HOME
	}
}
# Alpha already has these two renamed internally
# they need their argument packaged as a list!
;proc mkdir {dir} {
	oldMkdir [list $dir]
}
;proc rmdir {dir} {
	oldRmdir [list $dir]
}
} else {
;proc mkdir {dir} {
	file mkdir $dir
}

}

# not really a 'fix', but it's much more efficient in many places if
# you can set the mode of a window in advance  ---- else you switch
# modes twice on opening the window!  This version of 'new' has a new
# flag '-m' which lets you set the mode.  It also returns the name
# of the window which was really opened.
rename new __new
;proc new {args} {
	set i 0
	while {[set arg [lindex $args $i]] != ""} {
		incr i
		switch -- $arg {
			"-n" { 
				set name [lindex $args $i]
				incr i
			}
			"-g" { incr i 4 }
			"-m" { 
				set mode [lindex $args $i]
				set mi $i
				incr i
			}
			default {
				error "Incorrect args to 'new'"
			}
		}
	}
	if [info exists mode] {
		set args [lreplace $args [expr $mi -1] $mi]
		if ![info exists name] { set name "Untitled" }
		if {[lsearch -exact [winNames -f] $name] != -1} {
			set i 2
			while {[lsearch -exact [winNames -f] "$name <$i>"] != -1} {
				incr i
			}
			append name " <${i}>"
		}
		global win::Modes
		set win::Modes($name) $mode
	}
	eval __new $args
	if [info exists mode] { return $name } else {
		return [win::Current]
	}
}

# If the position to blink is offscreen, show a message with context
rename blink __blink
;proc blink {pos} {
	__blink $pos
	getWinInfo w
	set topl $w(currline)
	set endl [expr $topl + $w(linesdisp)]
	set bl [lindex [posToRowCol $pos] 0]
	if {$bl < $topl || $bl >= $endl} {
		message "Matching '[getText [lineStart $pos] [expr $pos +1]]'"
	}
}

# bring to front does nothing if already foremost 
# (the original calls activateHook, changeMode....)
rename bringToFront __bringToFront
;proc bringToFront {name} {
	global win::Current
	if {[file tail $name] != [file tail ${win::Current}]} { __bringToFront $name }
}

# keep window vertical position the same
rename revert __revert
;proc revert {} {
	getWinInfo w
	set topl $w(currline)
	__revert
	display [rowColToPos $topl 0]
}

# This routine is broken
rename specToPathName __specToPathName

# so we hack our way around it

## 
# -------------------------------------------------------------------------
# 
# "specToPathName" --
# 
#  given an FSSpec in hex form (as that returned by aebuild, for instance, 
#  see the definition of thinkFileName in :Tcl:SystemCode:think.tcl), 
#  return a complete pathname.
#  
# -------------------------------------------------------------------------
##
proc specToPathName {fss} {
   return [aevt::TEXT [aevt::keywordValue ---- \
	 [lindex [aevt::event \
	 [AEBuild -r 'MACS' core getd ---- \
	 "obj {form:alis, want:file, from:'null'(), seld:alis($fss)}" \
	 rtyp TEXT]] \
	 2]]]
}
